/**
  *  \file client/widgets/navwidget.h
  *  \brief Class WShipNavigationChart
  *
  *  (c) 2001-2015 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
#ifndef PCC_V2_CLIENT_WIDGETS_NAVWIDGET_H
#define PCC_V2_CLIENT_WIDGETS_NAVWIDGET_H

#include "client/wscanchart.h"
#include "game/coord.h"
#include "game/lock.h"

class GUniverse;
class GShip;
class UIAbstractButton;

/** Navigation chart widget.
    Implements a scanner chart with a few navigation-related functions:
    - warp and HYP circles
    - chunnel setup
    - keyboard recentering and ship/planet scanning */
class WShipNavigationChart : public WScannerChartWidget {
    /* Position history */
    enum { MAX_OLD_POS = 20 };
    GPoint old_pos[MAX_OLD_POS];
    int old_pos_head;
    int old_pos_count;

    /* Chunnel operation */
    bool chunnel_mode;
    UIAbstractButton* chunnel_button;
    const GShip* chunnel_ship;

    /* Warp circles */
    bool circle_hyp;
    int circle_radius;

    /* Ship */
    GUniverse& univ;

    /* Alternative warp circles */
    GPoint alt_circles;

    void drawCircles(GfxCanvas& can, GPoint pt);

 public:
    WShipNavigationChart(GUniverse& univ);

    void setCircleRadius(bool hyp, int r);
    void setChunnelShip(const GShip& ship);
    void setInitialZoom();

    void drawPre(GfxCanvas& can);
    bool handleEvent(const UIEvent& event, bool second_pass);
    void setChunnelButton(UIAbstractButton* btn);
    void toggleChunnel();
    bool isChunnelMode() const;
    void doItemLock(GPoint clicked, TLockMode mode, bool marked_only, bool optimize_warp);

    Signal1<int> sig_ship_select;
};

bool isValidChunnelMate(const GShip& initiator, const GShip& mate);

#endif
