/**
  *  \file game/actions/basefix.h
  *  \brief Starbase Action: Fix/Recycle
  *
  *  (c) 2001-2015 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
#ifndef PCC_V2_GAME_ACTIONS_BASEFIX_H
#define PCC_V2_GAME_ACTIONS_BASEFIX_H

#include "game/planet.h"
#include "game/error.h"

class GUniverse;
class GObjectList;

/** Starbase Action: Fix/Recycle.
    This contains the logic to determine what ships can be fixed or recycled. */
class GStarbaseFixRecycleAction {
 public:
    enum {
        /** I can fix this ship. */
        CanFixFlag           = (1 << sya_Fix),

        /** I can recycle this ship. */
        CanRecycleFlag       = (1 << sya_Recycle),

        /** I cannot do anything to this ship although it is at my place. */
        SamePositionFlag     = (1 << sya_None)
    };

    GStarbaseFixRecycleAction(GUniverse& univ, GPlanet& planet);

    bool setAction(TShipyardAction action, int ship_id);

    int  getValidBaseActions() const;
    void enumerateShipsFor(TShipyardAction action, GObjectList& list) const;
    int  getValidBaseActionsForShip(int ship_id) const;

 private:
    GUniverse& univ;
    GPlanet&   planet;
};

GError checkBaseAction(TShipyardAction action, GUniverse& univ, int ship_id, int planet_id);

#endif
