/**
  *  A PlayVCR / FLAK Winplan Wrapper
  *
  *  Hacked together by Stefan Reuther, using ideas from Christian
  *  Brckner's earlier implementation.
  */

#include <string>
#include <fstream>
#include <windows.h>
#include <ctype.h>
#include <stdio.h>

std::string opt_playvcr ("playvcr.exe");
std::string opt_playvcrparameters ("");
std::string opt_winplan ("winplano.exe");
std::string opt_vcrtitle ("Video Combat Recorder");
std::string opt_byetitle ("Bye Bye ");    // This name is used by the shareware version. Note the trailing space!
std::string opt_byetitle1 ("Bye Bye");    // This name is used by the 3.52.025

/** Convert integer to string. */
std::string
itoa(int i)
{
    char tmp[20];
    sprintf(tmp, "%d", i);
    return std::string(tmp);
}

/** Trim left whitespace off a string.
    \param s [in/out] the string
    \returns s */
std::string&
strLTrim(std::string& s)
{
    std::string::size_type n = 0;
    while (n < s.size() && isspace(static_cast<unsigned char>(s[n])))
        ++n;
    if (n)
        s.erase(0, n);
    return s;
}

/** Trim right whitespace off a string.
    \param s [in/out] the string
    \returns s */
std::string&
strRTrim(std::string& s)
{
    std::string::size_type n = s.size();
    while (n > 0 && isspace(static_cast<unsigned char>(s[n-1])))
        --n;
    if (n != s.size())
        s.erase(n);
    return s;
}

/** Read configuration file. */
void
readConfigFile()
{
    std::ifstream in("vcr.cfg");
    if (!in)
        return;
    std::string line;
    while (std::getline(in, line)) {
        // preprocess
        strLTrim(line);
        strRTrim(line);
        if (!line.size() || line[0] == '#' || line[0] == ';')
            continue;

        // find delimiter
        std::string::size_type n = line.find('=');
        if (n == line.npos)
            continue;

        // split it
        std::string keyword (line, 0, n);
        std::string value (line, n+1);
        if (keyword == "playvcr")
            opt_playvcr = value;
        else if (keyword == "playvcrparameters")
            opt_playvcrparameters = value;
        else if (keyword == "winplan")
            opt_winplan = value;
        else if (keyword == "title.vcr")
            opt_vcrtitle = value;
        else if (keyword == "title.bye")
            opt_byetitle = value;
        else if (keyword == "title.bye1")
            opt_byetitle1 = value;
    }
}

/** Launch Winplan. */
void
runWinplan(int show, HANDLE& process)
{
    SHELLEXECUTEINFO winplan = {0};
    winplan.cbSize=sizeof(SHELLEXECUTEINFO);
    winplan.lpFile=opt_winplan.c_str();
    winplan.nShow=show;
    winplan.fMask = SEE_MASK_NOCLOSEPROCESS;
    ShellExecuteEx(&winplan);
    process = winplan.hProcess;
}

/** Log something. */
// void
// log(std::string line)
// {
//     //std::cout << line << std::endl;
//     //MessageBox(0, line.c_str(), "status", MB_OK);
// }
#define log(x)

/** File status. \see testFile */
enum FileStatus {
    FileExists   = 1,
    FileIsLocked = 2,
    FileError    = 3
};

/** Test file's status.
    \param name [in] file name
    \param time [out] file modification or access time, whichever is more
                recent. Parameter can be null if you don't want the time.
    \return FileStatus */
FileStatus
testFile(std::string name, FILETIME* time)
{
    HANDLE h = CreateFile(name.c_str(), GENERIC_READ, FILE_SHARE_READ, 0, OPEN_EXISTING, 0, 0);
    if (h == INVALID_HANDLE_VALUE) {
        if (GetLastError() == ERROR_SHARING_VIOLATION)
            return FileIsLocked;
        else
            return FileError;
    } else {
        if (time) {
            BY_HANDLE_FILE_INFORMATION fi;
            if (GetFileInformationByHandle(h, &fi)) {
                // return the file's most recent time
                if (CompareFileTime(&fi.ftLastWriteTime, &fi.ftLastAccessTime) > 0)
                    *time = fi.ftLastWriteTime;
                else
                    *time = fi.ftLastAccessTime;
            } else {
                // unable to get information
                time->dwLowDateTime = 0;
                time->dwHighDateTime = 0;
            }
        }
        CloseHandle(h);
        return FileExists;
    }
}

/** Figure out the game number which was last opened in Winplan. */
int
findGameNumber()
{
    char game;
    std::ifstream in ("gamestat.dat", std::ios::in | std::ios::binary);
    if (!in) {
        MessageBox(0, "Unable to open 'gamestat.dat'.", "VCR Error", MB_ICONSTOP | MB_OK);
        return 0;
    }
    in.seekg(1649);
    if (!in.get(game)) {
        MessageBox(0, "Unable to read 'gamestat.dat'.", "VCR Error", MB_ICONSTOP | MB_OK);
        return 0;
    }
    in.close();
    if (game <= 0 || game > 8) {
        MessageBox(0, "I do not understand your 'gamestat.dat'.", "VCR Error", MB_ICONSTOP | MB_OK);
        return 0;
    }
    return game;
}

/** Main entry to run the VCR. */
void
runVcr()
{
    // figure out game number
    int game = findGameNumber();
    if (!game)
        return;

    std::string gamepath ("vpwork");
    gamepath += itoa(game);

    log("gamestat.dat says: gamepath '" + gamepath + "'");

    // now, find the player. First, check maybe there is only one?
    int player = 0;
    bool there[12];
    for (int i = 1; i <= 11; ++i) {
        if (testFile(gamepath + "/vcr" + itoa(i) + ".dat", 0) != FileError) {
            there[i] = true;
            log("found player " + itoa(i) + " file");
            if (player == 0)
                player = i;
            else
                player = -1;
        } else {
            there[i] = false;
        }
    }

    if (player == 0) {
        MessageBox(0, "Unable to find any VCR files.", "VCR Error", MB_ICONSTOP | MB_OK);
        return;
    }

    if (player < 0) {
        // ambiguous. Figure out which file was most recently
        // accessed.
        static const char* const basenames[] = { "ship", "pdata", "bdata", 0 };
        FILETIME youngest;
        log("situation ambiguous, checking file times");
        for (int i = 1; i <= 11; ++i) {
            if (!there[i])
                continue;
            for (int f = 0; basenames[f] != 0; ++f) {
                std::string this_name = gamepath + "\\" + basenames[f] + itoa(i) + ".dat";
                FILETIME this_time;
                switch (testFile(this_name, &this_time)) {
                 case FileExists:
                    // pick this player if we don't have one yet, or
                    // if the file is younger
                    if (player < 0 || CompareFileTime(&this_time, &youngest) > 0) {
                        player = i;
                        youngest = this_time;
                        log("  youngest so far '" + this_name + "'");
                    }
                    break;                    
                 case FileIsLocked:
                    // sharing violation
                    log("got a sharing violation on '" + this_name + "', assuming winplan has it open");
                    player = i;
                    goto found;
                 case FileError:
                    break;
                }
            }
        }
    found:;
    }

    if (player <= 0) {
        MessageBox(0, "Sorry, I am unable to make any sense of your game directory.", "VCR Error", MB_ICONSTOP | MB_OK);
        return;
    }

    // check whether it is FLAK
    bool flak = (testFile(gamepath + "\\flak" + itoa(player) + ".dat", 0) != FileError);

    // we now know everything we need, start the viewer
    log(">> gamepath = " + gamepath + ", player = " + itoa(player) + ", flak = " + itoa(flak) + " <<");

    std::string params = opt_playvcrparameters;
    if (params.size())
        params.append(" ");
    params.append(gamepath);
    params.append(flak ? "\\flak" : "\\vcr");
    params.append(itoa(player));
    params.append(".dat");

    SHELLEXECUTEINFO si={0};
    si.cbSize = sizeof(SHELLEXECUTEINFO);
    si.lpFile = opt_playvcr.c_str();
    si.lpParameters = params.c_str();
    si.nShow = SW_SHOW;
    ShellExecuteEx(&si);
}

/** Main entry point. */
void
realMain(int show)
{
    HANDLE procnr;
    readConfigFile();
    runWinplan(show, procnr);
    while (1) {
        HWND w;
        w = FindWindow(0, opt_byetitle.c_str());
        if (w != 0) {
            log("Terminating because Bye window found");
            break;
        }

        w = FindWindow(0, opt_byetitle1.c_str());
        if (w != 0) {
            log("Terminating because Bye1 window found");
            break;
        }

        w = FindWindow(0, opt_vcrtitle.c_str());
        if (w != 0) {
            PostMessage(w, WM_CLOSE, 0, 0);
            runVcr();
        }

        /* This would be the cleanest way to detect when Winplan has
           exited, but does not seem to work. We exit too early with
           this implementation. */
        // DWORD status;
        // if (GetExitCodeProcess(procnr, &status) && status != STILL_ACTIVE) {
        //     log("Terminating because process exited");
        //     break;
        // }

        Sleep(100);
    }
}

//// Entry points. Depending whether this is compiled as console or
//// GUI app, one of these is chosen.

int
main()
{
    realMain(SW_SHOW);
    return 0;
}

int WINAPI
WinMain(HINSTANCE /*hinst*/, HINSTANCE /*hprev*/, LPSTR /*cmdline*/, int show)
{
    realMain(show);
    return 0;
}
