/**
  *  \file int/if/planetif.h
  *  \brief Interpreter Game Interface: Planets
  *
  *  (c) 2001-2015 Stefan Reuther <Streu@gmx.de>
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of file `COPYING' that comes with the
  *  source code.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
#ifndef PCC_V2_INT_IF_PLANETIF_H
#define PCC_V2_INT_IF_PLANETIF_H

#include "cpluslib/types.h"
#include "int/context.h"
#include "cpluslib/string.h"
#include "int/if/objcontext.h"

class IntValue;
class IntArgBlock;
class IntExecutionContext;
class GPlanet;

IntValue*   IFPlanetGet(IntArgBlock& args);
IntContext* IFPlanetMake();
int32_t     IFPlanetDim(int32_t dim);

/** Ship context. */
class IntPlanetContext : public IntObjectContext {
 public:
    IntPlanetContext(int pid);
    ~IntPlanetContext();

    // IntContext:
    int32_t   lookup(const IntNameQuery& name);
    void      set(int32_t index, IntValue* value);
    IntValue* get(int32_t index);
    IntContext* clone();
    string_t  toString(bool readable);
    void      store(IntTagNode& sv, Stream& aux);

    // IntPropertyEnumerator:
    void      enumProperties(IntPropertyAcceptor& acceptor);
};

enum IntPlanetProperty {
    ippBaseBuildFlag,
    ippBaseDefenseWanted,
    ippBaseFlag,
    ippBaseStr,
    ippColonistChange,
    ippColonistChangeStr,
    ippColonistHappy,
    ippColonistHappyStr,
    ippColonistSupported,
    ippColonistTax,
    ippColonistTaxIncome,
    ippColonists,
    ippDefense,
    ippDefenseMax,
    ippDefenseWanted,
    ippDensityD,
    ippDensityM,
    ippDensityN,
    ippDensityT,
    ippFCode,
    ippFactories,
    ippFactoriesMax,
    ippFactoriesWanted,
    ippGroundD,
    ippGroundM,
    ippGroundN,
    ippGroundT,
    ippId,
    ippIndustry,
    ippIndustryCode,
    ippLevel,
    ippLocX,
    ippLocY,
    ippMarked,
    ippMinedD,
    ippMinedM,
    ippMinedN,
    ippMinedStr,
    ippMinedT,
    ippMines,
    ippMinesMax,
    ippMinesWanted,
    ippMoney,
    ippName,
    ippNativeChange,
    ippNativeChangeStr,
    ippNativeGov,
    ippNativeGovCode,
    ippNativeHappy,
    ippNativeHappyStr,
    ippNativeRace,
    ippNativeRaceCode,
    ippNativeTax,
    ippNativeTaxBase,
    ippNativeTaxIncome,
    ippNativeTaxMax,
    ippNatives,
    ippOrbitingEnemies,
    ippOrbitingOwn,
    ippOrbitingShips,
    ippPlayed,
    ippScore,
    ippSupplies,
    ippTask,
    ippTaskBase,
    ippTemp,
    ippTempStr,
    ippTypeChar,
    ippTypeStr
};

enum IntPlanetMethod {
    ipmMark,                    // 0
    ipmUnmark,                  // 1
    ipmSetComment,              // 2
    ipmFixShip,                 // 3
    ipmRecycleShip,             // 4
    ipmBuildBase,               // 5
    ipmAutoBuild,               // 6
    ipmBuildDefense,            // 7
    ipmBuildFactories,          // 8
    ipmBuildMines,              // 9
    ipmSetColonistTax,          // 10
    ipmSetNativeTax,            // 11
    ipmSetFCode,                // 12
    ipmSetMission,              // 13
    ipmBuildBaseDefense,        // 14
    ipmSetTech,                 // 15
    ipmBuildFighters,           // 16
    ipmBuildEngines,            // 17
    ipmBuildHulls,              // 18
    ipmBuildLaunchers,          // 19
    ipmBuildBeams,              // 20
    ipmBuildTorps,              // 21
    ipmSellSupplies,            // 22
    ipmBuildShip,               // 23
    ipmCargoTransfer,           // 24
    ipmAutoTaxColonists,        // 25
    ipmAutoTaxNatives           // 26
};

IntValue* getPlanetProperty(const GPlanet& pl, GUniverse& univ, IntPlanetProperty ipp);
void      setPlanetProperty(GPlanet& pl, IntPlanetProperty ipp, IntValue* value);
void      callPlanetMethod(GPlanet& pl, GUniverse& univ, IntPlanetMethod ipm, IntExecutionContext& exc, IntArgBlock& args);

#endif
